@@ -3,6 +3,7 @@
 import argparse
 import collections
 import copy
+import datetime
 import json
 import os
 import random
@@ -1136,12 +1137,22 @@ class RecordingHandle:
 
     timeout = None
 
+    # max length of records per output file.
+    # If a recording is continued longer than this, all information recorded so
+    # far is saved at self.file_path.%Y-%m-%d-%H-%M-%S/ directory.
+    #
+    # default to 3600 seconds (1 hour).
+    # TODO: Add command line option for setting this.
+    max_seconds_per_file = 3600
+    max_seconds_per_file_exceeded = None
+
     def __init__(self, tracepoints, file_path, file_format, file_permission,
                  monitoring_intervals,
                  do_profile,
                  kdamonds, add_child_tasks, record_mem_footprint,
                  record_vmas, record_proc_stats, timeout, snapshot_request,
-                 snapshot_interval_sec, snapshot_count):
+                 snapshot_interval_sec, snapshot_count,
+                 max_seconds_per_file=3600):
         self.tracepoints = tracepoints
         self.file_path = file_path
         self.file_format = file_format
@@ -1166,6 +1177,8 @@ class RecordingHandle:
         self.snapshot_interval_sec = snapshot_interval_sec
         self.snapshot_count = snapshot_count
 
+        self.max_seconds_per_file = max_seconds_per_file
+
     def will_take_awhile(self):
         if self.snapshot_interval_sec == 0 and self.snapshot_count < 5:
             return False
@@ -1183,12 +1196,7 @@ def tracepoint_supported(tracepoint):
             return True
     return False
 
-def start_recording(handle):
-    kdamonds_file_path = '%s.kdamonds' % handle.file_path
-    with open(kdamonds_file_path, 'w') as f:
-        json.dump([k.to_kvpairs() for k in handle.kdamonds], f, indent=4)
-    os.chmod(kdamonds_file_path, handle.file_permission)
-
+def start_recording_perf(handle):
     if handle.tracepoints is not None:
         tracepoints_option = []
         for tracepoint in handle.tracepoints:
@@ -1201,7 +1209,11 @@ def start_recording(handle):
         cmd = [PERF, 'record', '-o', '%s.profile' % handle.file_path]
         handle.perf_profile_pipe = subprocess.Popen(cmd)
 
+def start_recording(handle):
+    start_recording_perf(handle)
+
     start_time = time.time()
+    last_output_saved_time = start_time
     nr_snapshots_to_take = handle.snapshot_count
     if handle.snapshot_interval_sec:
         sleep_time = handle.snapshot_interval_sec
@@ -1210,7 +1222,7 @@ def start_recording(handle):
     while (poll_target_pids(handle.kdamonds) or
            _damon.any_kdamond_running()):
         if handle.add_child_tasks is True:
-            _damon.add_childs_target(handle.kdamonds)
+            _damon.add_commit_vaddr_child_targets(handle.kdamonds)
 
         if handle.mem_footprint_snapshots is not None:
             record_mem_footprint(handle.kdamonds,
@@ -1224,6 +1236,22 @@ def start_recording(handle):
         if (handle.timeout is not None and
             time.time() - start_time >= handle.timeout):
             break
+        if (handle.max_seconds_per_file is not None and time.time() -
+            last_output_saved_time >= handle.max_seconds_per_file):
+            last_output_saved_time = time.time()
+            handle.max_seconds_per_file_exceeded = True
+
+            save_recording_outputs(handle, handle.file_path)
+
+            if handle.mem_footprint_snapshots is not None:
+                handle.mem_footprint_snapshots = []
+            if handle.vmas_snapshots is not None:
+                handle.vmas_snapshots = []
+            if handle.proc_stats is not None:
+                handle.proc_stats = []
+            if handle.snapshot_records is not None:
+                handle.snapshot_records = []
+            start_recording_perf(handle)
 
         if handle.snapshot_request:
             if handle.snapshot_records is None:
@@ -1240,7 +1268,19 @@ def start_recording(handle):
 
         time.sleep(sleep_time)
 
-def finish_recording(handle):
+def save_recording_outputs(handle, file_path):
+    if handle.max_seconds_per_file_exceeded is True:
+        dirname = '%s.%s' % (
+                handle.file_path,
+                datetime.datetime.now().strftime('%Y-%m-%d-%H-%M-%S'))
+        os.mkdir(dirname)
+        file_path = os.path.join(dirname, file_path)
+
+    kdamonds_file_path = '%s.kdamonds' % file_path
+    with open(kdamonds_file_path, 'w') as f:
+        json.dump([k.to_kvpairs() for k in handle.kdamonds], f, indent=4)
+    os.chmod(kdamonds_file_path, handle.file_permission)
+
     if handle.perf_pipe:
         try:
             handle.perf_pipe.send_signal(signal.SIGINT)
@@ -1248,18 +1288,19 @@ def finish_recording(handle):
         except:
             # perf might already finished
             pass
+        os.rename(handle.file_path, file_path)
 
         if handle.file_format == file_type_perf_data:
-            os.chmod(handle.file_path, handle.file_permission)
+            os.chmod(file_path, handle.file_permission)
         else:
-            err = update_records_file(handle.file_path, handle.file_format,
+            err = update_records_file(file_path, handle.file_format,
                     handle.file_permission, handle.monitoring_intervals)
             if err is not None:
                 print('converting format from perf_data to %s failed (%s)' %
                         (handle.file_format, err))
 
     if handle.snapshot_records:
-        write_damon_records(handle.snapshot_records, handle.file_path,
+        write_damon_records(handle.snapshot_records, file_path,
                             handle.file_format, handle.file_permission)
 
     if handle.perf_profile_pipe is not None:
@@ -1268,19 +1309,23 @@ def finish_recording(handle):
         except:
             # perf might already finished
             pass
-        os.chmod('%s.profile' % handle.file_path, handle.file_permission)
+        os.rename('%s.profile' % handle.file_path, '%s.profile' % file_path)
+        os.chmod('%s.profile' % file_path, handle.file_permission)
 
     if handle.mem_footprint_snapshots is not None:
         save_mem_footprint(
                 handle.mem_footprint_snapshots,
-                '%s.mem_footprint' % handle.file_path, handle.file_permission)
+                '%s.mem_footprint' % file_path, handle.file_permission)
     if handle.vmas_snapshots is not None:
-        save_proc_vmas(handle.vmas_snapshots, '%s.vmas' % handle.file_path,
+        save_proc_vmas(handle.vmas_snapshots, '%s.vmas' % file_path,
                        handle.file_permission)
     if handle.proc_stats is not None:
-        save_proc_stats(handle.proc_stats, '%s.proc_stats' % handle.file_path,
+        save_proc_stats(handle.proc_stats, '%s.proc_stats' % file_path,
                         handle.file_permission)
 
+def finish_recording(handle):
+    save_recording_outputs(handle, handle.file_path)
+
 # for snapshot
 
 def current_kdamonds_interval_updated():
