@@ -393,17 +393,21 @@ class DamonRegion:
 
 class DamonTarget:
     pid = None
+    obsolete = None
     regions = None
     context = None
 
-    def __init__(self, pid, regions):
+    def __init__(self, pid, regions, obsolete=False):
         self.pid = pid
         self.regions = regions
+        self.obsolete = _damo_fmt_str.text_to_bool(obsolete)
 
     def to_str(self, raw):
         lines = []
         if self.pid is not None:
             lines.append('pid: %s' % self.pid)
+        if self.obsolete is True:
+            line.sappend('(obsolete)')
         for region in self.regions:
             lines.append('region %s' % region.to_str(raw))
         return '\n'.join(lines)
@@ -417,11 +421,15 @@ class DamonTarget:
     @classmethod
     def from_kvpairs(cls, kvpairs):
         regions = [DamonRegion.from_kvpairs(kvp) for kvp in kvpairs['regions']]
-        return DamonTarget(kvpairs['pid'], regions)
+        obsolete = False
+        if 'obsolete' in kvpairs:
+            obsolete = kvpairs['obsolete']
+        return DamonTarget(kvpairs['pid'], regions, obsolete=obsolete)
 
     def to_kvpairs(self, raw=False):
         kvp = collections.OrderedDict()
         kvp['pid'] = self.pid
+        kvp['obsolete'] = self.obsolete
         kvp['regions'] = [r.to_kvpairs(raw) for r in self.regions]
         return kvp
 
@@ -573,10 +581,13 @@ qgoal_user_input = 'user_input'
 qgoal_some_mem_psi_us = 'some_mem_psi_us'
 qgoal_node_mem_used_bp = 'node_mem_used_bp'
 qgoal_node_mem_free_bp = 'node_mem_free_bp'
+qgoal_node_memcg_used_bp = 'node_memcg_used_bp'
+qgoal_node_memcg_free_bp = 'node_memcg_free_bp'
 qgoal_active_mem_bp = 'active_mem_bp'
 qgoal_inactive_mem_bp = 'inactive_mem_bp'
 qgoal_metrics = [qgoal_user_input, qgoal_some_mem_psi_us,
                  qgoal_node_mem_used_bp, qgoal_node_mem_free_bp,
+                 qgoal_node_memcg_used_bp, qgoal_node_memcg_free_bp,
                  qgoal_active_mem_bp, qgoal_inactive_mem_bp]
 
 class DamosQuotaGoal:
@@ -584,16 +595,19 @@ class DamosQuotaGoal:
     target_value = None
     current_value = None
     nid = None
+    memcg_path = None
     quotas = None
 
     def __init__(self, metric=qgoal_user_input,
-                 target_value='0', current_value='0', nid=None):
+                 target_value='0', current_value='0', nid=None,
+                 memcg_path=None):
         if not metric in qgoal_metrics:
             raise Exception('unsupported DAMOS quota goal metric')
         self.metric = metric
         if metric == qgoal_some_mem_psi_us:
             self.target_value = _damo_fmt_str.text_to_us(target_value)
         elif metric in [qgoal_node_mem_used_bp, qgoal_node_mem_free_bp,
+                        qgoal_node_memcg_used_bp, qgoal_node_memcg_free_bp,
                         qgoal_active_mem_bp, qgoal_inactive_mem_bp]:
             self.target_value = _damo_fmt_str.text_to_bp(target_value)
         else:
@@ -603,6 +617,7 @@ class DamosQuotaGoal:
             self.nid = None
         else:
             self.nid = _damo_fmt_str.text_to_nr(nid)
+        self.memcg_path = memcg_path
 
     @classmethod
     def metric_require_nid(cls, metric):
@@ -611,11 +626,24 @@ class DamosQuotaGoal:
     def has_nid(self):
         return DamosQuotaGoal.metric_require_nid(self.metric)
 
+    @classmethod
+    def metric_require_memcg_path(cls, metric):
+        return metric in [qgoal_node_memcg_used_bp, qgoal_node_memcg_free_bp]
+
+    def has_memcg_path(self):
+        return DamosQuotaGoal.metric_require_memcg_path(self.metric)
+
     def to_str(self, raw):
         metric_str = self.metric
+        additional_words = []
         if self.has_nid():
-            metric_str = '%s (nid %s)' % (
-                    metric_str, _damo_fmt_str.format_nr(self.nid, raw))
+            additional_words.append('nid %s' %
+                                    _damo_fmt_str.format_nr(self.nid, raw))
+        if self.has_memcg_path():
+            additional_words.append('memcg %s' % self.memcg_path)
+
+        if len(additional_words) > 0:
+            metric_str = '%s (%s)' % (metric_str, ', '.join(additional_words))
         return 'metric %s target %s current %s' % (
                 metric_str,
                 _damo_fmt_str.format_nr(self.target_value, raw),
@@ -627,6 +655,7 @@ class DamosQuotaGoal:
     def __eq__(self, other):
         return (type(self) == type(other) and self.metric == other.metric and
                 self.nid == other.nid and
+                self.memcg_path == other.memcg_path and
                 self.target_value == other.target_value and
                 self.current_value == other.current_value)
 
@@ -637,8 +666,13 @@ class DamosQuotaGoal:
             # later.
             return DamosQuotaGoal(target_value=kv['target_value_bp'],
                                   current_value=kv['current_value_bp'])
+        if 'memcg_path' in kv:
+            memcg_path = kv['memcg_path']
+        else:
+            memcg_path = None
         return DamosQuotaGoal(
                 metric=kv['metric'], nid=kv['nid'] if 'nid' in kv else None,
+                memcg_path=memcg_path,
                 target_value=kv['target_value'],
                 current_value=kv['current_value'])
 
@@ -647,6 +681,7 @@ class DamosQuotaGoal:
             ('metric', self.metric),
             ('nid', _damo_fmt_str.format_nr(self.nid, raw)
              if self.nid is not None else None),
+            ('memcg_path', self.memcg_path),
             ('target_value', _damo_fmt_str.format_nr(self.target_value,
                 raw)),
             ('current_value', _damo_fmt_str.format_nr(self.current_value,
@@ -1242,6 +1277,9 @@ class DamonCtx:
 def target_has_pid(ops):
     return ops in ['vaddr', 'fvaddr']
 
+def target_regions_fixed(ops):
+    return ops in ['fvaddr', 'paddr']
+
 class Kdamond:
     state = None
     pid = None
@@ -1341,7 +1379,7 @@ import damo_version
 # similar issue could happen in future.
 #
 # Hence, damo features for online DAMON control or snapshot cannot make
-# feature_supprots correctly.  And repeated feature check is waste of time,
+# feature_supports correctly.  And repeated feature check is waste of time,
 # anyway.
 #
 # To work around, ask features that would run while DAMON is not running to
@@ -1392,6 +1430,10 @@ features = ['record',       # was in DAMON patchset, but not merged in mainline
                                         # merged in v6.16-rc1
             'schemes_dests',            # merged in v6.17-rc1
             'sysfs_refresh_ms',         # merged in v6.17-rc1
+            'addr_unit',                # merged in v6.18-rc1
+            'schemes_quota_goal_node_memcg_used_free',
+                                        # merged in mm, expectred for 6.19-rc1
+            'obsolete_target',          # merged in mm, expected for 6.19-rc1
             'ops_attrs',                # hacking on damon/next
             ]
 
@@ -1483,6 +1525,13 @@ def write_feature_supports_file():
     with open(feature_supports_file_path, 'w') as f:
         json.dump(to_save, f, indent=4, sort_keys=True)
 
+def rm_feature_supports_file():
+    try:
+        os.remove(feature_supports_file_path)
+    except Exception as e:
+        return '%s' % e
+    return None
+
 def feature_supported(feature):
     return _damon_fs.feature_supported(feature)
 
@@ -1587,6 +1636,23 @@ def commit_quota_goals(kdamond_idxs):
         return 'debugfs interface does not support commit_quota_goals()'
     return _damon_fs.commit_quota_goals(kdamond_idxs)
 
+def cleanup_obsolete_targets(kdamonds):
+    '''
+    When obsolete targets are committed to DAMON, DAMON removes the obsolete
+    ones inside kernel, but leave the sysfs files as is.  Cleanup the obsolete
+    sysfs files by staging only non-obsolete targets again.
+    '''
+    need_cleanup = False
+    for kd in kdamonds:
+        for ctx in kd.contexts:
+            for target in ctx.targets:
+                if target.obsolete:
+                    need_cleanup = True
+            ctx.targets = [t for t in ctx.targets if not t.obsolete]
+    if not need_cleanup:
+        return None
+    return stage_kdamonds(kdamonds)
+
 def commit(kdamonds, commit_quota_goals_only=False, commit_targets_only=False):
     if not commit_quota_goals_only and not commit_targets_only:
         err = stage_kdamonds(kdamonds)
@@ -1608,6 +1674,11 @@ def commit(kdamonds, commit_quota_goals_only=False, commit_targets_only=False):
     err = commit_staged(kdamond_idxs)
     if err:
         return 'commit staged updates filed (%s)' % err
+
+    err = cleanup_obsolete_targets(kdamonds)
+    if err is not None:
+        return 'obsolte targets cleanup failed (%s)' % err
+
     return None
 
 def update_tuned_intervals(kdamond_idxs=None):
@@ -1695,11 +1766,35 @@ def pid_running(pid):
     except:
         return False
 
-def add_childs_target(kdamonds):
-    # TODO: Support multiple kdamonds
-    if not target_has_pid(kdamonds[0].contexts[0].ops):
-        return
-    old_targets = kdamonds[0].contexts[0].targets
+def add_vaddr_child_targets_with_obsolete_support(ctx):
+    changes_made = False
+    orig_targets = ctx.targets
+    updated_targets = []
+    child_targets = []
+    for orig_target in orig_targets:
+        updated_targets.append(DamonTarget(pid=orig_target.pid, regions=[]))
+        if not pid_running(orig_target.pid):
+            updated_targets[-1].obsolete = True
+            changes_made = True
+        for child_pid in get_childs_pids('%s' % orig_target.pid):
+            child_targets.append(DamonTarget(pid=child_pid, regions=[]))
+            changes_made = True
+    if changes_made:
+        ctx.targets = updated_targets + child_targets
+    return changes_made, orig_targets
+
+def add_vaddr_child_targets(ctx):
+    '''
+    Returns whether a change is made, and old targets list if a change was made
+    '''
+    if not target_has_pid(ctx.ops):
+        return False, None
+    if target_regions_fixed(ctx.ops):
+        return False, None
+    if feature_supported('obsolete_target'):
+        return add_vaddr_child_targets_with_obsolete_support(ctx)
+
+    old_targets = ctx.targets
     current_target_pids = []
     new_target_pids = []
 
@@ -1730,20 +1825,27 @@ def add_childs_target(kdamonds):
             new_target_pids.append(child_pid)
 
     if set(new_target_pids) == set(current_target_pids):
-        return None
-    # Commit the new set of targets.
-    #
+        return False, None
+
     # Set no target region because it will instruct DAMON to keep current
     # monitoring results (regions with nr_accesses, age, etc) for the target if
     # exists.
     #
     # TODO: adjust the orders of the new targets so that only appropriate past
     # monitoring results are inherited.
-    kdamonds[0].contexts[0].targets = [DamonTarget(pid = p, regions=[])
+    ctx.targets = [DamonTarget(pid = p, regions=[])
                                        for p in new_target_pids]
+    return True, old_targets
+
+def add_commit_vaddr_child_targets(kdamonds):
+    # TODO: Support multiple kdamonds
+    ctx = kdamonds[0].contexts[0]
+    need_commit, old_targets = add_vaddr_child_targets(ctx)
+    if not need_commit:
+        return
     err = commit(kdamonds, commit_targets_only=True)
     if err is not None:
-        kdamonds[0].contexts[0].targets = old_targets
+        ctx.targets = old_targets
         return 'commit failed (%s)' % err
 
     return None
