@@ -260,18 +260,29 @@ def damos_quotas_cons_arg(cmd_args):
 
 def damos_options_to_quota_goal(garg):
     # garg is the user inputs
-    # garg should be <metric> <target value> [<current value>|<nid>]
-    # [current value] is given for only 'user_input' <metric>
-    # [nid] is given for only node_mem_{used,free}_bp
-    if not len(garg) in [2, 3]:
+    # garg should be <metric> <target value> [<optional>...]
+    # for user_input, one optional argument for "current value" is given.
+    # for node_mem[cg]_{used,free}_bp, one optional argument for node id is
+    # given.
+    # for node_memcg_{used,free}_bp, one more optional argument for memcg path
+    # is given.
+    if not len(garg) in [2, 3, 4]:
         return None, 'Wrong --damos_quota_goal (%s)' % garg
     metric, target_value, optionals = garg[0], garg[1], garg[2:]
     current_value = 0
     nid = None
-    if _damon.DamosQuotaGoal.metric_require_nid(metric):
+    memcg_path = None
+    if metric in [_damon.qgoal_node_mem_used_bp,
+                  _damon.qgoal_node_mem_free_bp]:
         if len(optionals) != 1:
             return None, 'nid is not given or something else is given'
         nid = optionals[0]
+    elif metric in [_damon.qgoal_node_memcg_used_bp,
+                    _damon.qgoal_node_memcg_free_bp]:
+        if len(optionals) != 2:
+            return None, 'nid and memcg_path required'
+        nid = optionals[0]
+        memcg_path = optionals[1]
     elif metric == 'user_input':
         if len(optionals) != 1:
             return None, 'current value is not given or something else is given'
@@ -279,7 +290,8 @@ def damos_options_to_quota_goal(garg):
     try:
         return _damon.DamosQuotaGoal(
                 metric=metric, target_value=target_value,
-                current_value=current_value, nid=nid), None
+                current_value=current_value,
+                nid=nid, memcg_path=memcg_path), None
     except Exception as e:
         return None, 'DamosQuotaGoal creation fail (%s, %s)' % (garg, e)
 
@@ -478,10 +490,14 @@ def damon_target_for(args, idx, ops):
         return None, err
 
     try:
-        target = _damon.DamonTarget(args.target_pid[idx]
-                if _damon.target_has_pid(ops) else None, init_regions)
+        obsolete = False
+        if args.obsolete_targets is not None and idx in args.obsolete_targets:
+            obsolete = True
+        target = _damon.DamonTarget(
+                args.target_pid[idx] if _damon.target_has_pid(ops) else None,
+                init_regions, obsolete=obsolete)
     except Exception as e:
-        return 'Wrong \'--target_pid\' argument (%s)' % e
+        return None, 'Wrong \'--target_pid\' argument (%s)' % e
     return target, None
 
 def damon_ctx_for(args, idx):
@@ -585,8 +601,18 @@ def fillup_none_target_args(args):
             setattr(args, attr_name,
                     attr_val + [None] * (nr_targets - len(attr_val)))
 
+def valid_obsolete_targets_args(obsolete_targets, nr_targets):
+    if obsolete_targets is None:
+        return True
+    for target_idx in obsolete_targets:
+        if target_idx >= nr_targets:
+            return False
+    return True
+
 def gen_assign_targets(ctxs, args):
     nr_targets = get_nr_targets(args)
+    if not valid_obsolete_targets_args(args.obsolete_targets, nr_targets):
+        return 'Invalid --obsolete_targets'
     targets = []
     if args.nr_targets is None:
         if len(ctxs) != 1:
@@ -599,7 +625,7 @@ def gen_assign_targets(ctxs, args):
                 break
         target, err = damon_target_for(args, idx, ops)
         if err is not None:
-            return None, err
+            return err
         targets.append(target)
     if sum(args.nr_targets) != len(targets):
         return '--nr_targets and number of targets mismatch (%d != %d)' % (
@@ -938,6 +964,10 @@ def set_monitoring_argparser(parser, hide_help=False):
             '--nr_targets', metavar='<number>', nargs='+', type=int,
             help='number of monitoring targets for each context (in order)'
             if not hide_help else argparse.SUPPRESS)
+    parser.add_argument('--obsolete_targets', metavar='<target index>',
+                        nargs='+', type=int,
+                        help='obsolte targets'
+                        if not hide_help else argparse.SUPPRESS)
     parser.add_argument('--nr_ctxs', metavar='<number>', nargs='+', type=int,
                         help='number of contexts for each kdamond (in order)'
                         if not hide_help else argparse.SUPPRESS)
