@@ -282,6 +282,12 @@ def write_quota_goal_dir(dir_path, goal):
         if err is not None:
             return err
 
+    if goal.has_memcg_path():
+        err = _damo_fs.write_file(
+                os.path.join(dir_path, 'path'), '%s' % goal.memcg_path)
+        if err is not None:
+            return err
+
     err = _damo_fs.write_file(
             os.path.join(dir_path, 'target_value'),
             '%d' % goal.target_value)
@@ -477,6 +483,15 @@ def write_target_dir(dir_path, target):
         if err is not None:
             return err
 
+    obsolete_file = os.path.join(dir_path, 'obsolete_target')
+    if os.path.isfile(obsolete_file):
+        err = _damo_fs.write_file(
+                obsolete_file, '1' if target.obsolete else '0')
+        if err is not None:
+            return err
+    elif target.obsolete:
+        return 'obsolete_target unsupported'
+
     return write_target_regions_dir(
             os.path.join(dir_path, 'regions'), target.regions)
 
@@ -697,10 +712,15 @@ def files_content_to_quota_goals(files_content):
     goals = []
     for goal_kv in number_sorted_dirs(files_content):
         if 'target_metric' in goal_kv:
+            if 'path' in goal_kv:
+                memcg_path = goal_kv['path'].strip()
+            else:
+                memcg_path = None
             goals.append(
                     _damon.DamosQuotaGoal(
                         metric=goal_kv['target_metric'].strip(),
                         nid=goal_kv['nid'] if 'nid' in goal_kv else None,
+                        memcg_path=memcg_path,
                         target_value=goal_kv['target_value'],
                         current_value=goal_kv['current_value']))
         else:
@@ -830,8 +850,11 @@ def files_content_to_target(files_content):
         pid = int(files_content['pid_target'])
     except:
         pid = None
+    obsolete = False
+    if 'obsolete_target' in files_content:
+        obsolete = files_content['obsolete_target'].strip()
     regions = files_content_to_regions(files_content['regions'])
-    return _damon.DamonTarget(pid, regions)
+    return _damon.DamonTarget(pid, regions, obsolete=obsolete)
 
 def files_content_to_ops_attrs(files_content):
     use_reports = files_content['use_reports'].strip()
@@ -930,8 +953,6 @@ def commit_quota_goals(kdamond_idxs):
 feature_supports = None
 
 def feature_supported(feature):
-    if feature_supports == None:
-        update_supported_features()
     return feature_supports[feature]
 
 # sysfs was merged in v5.18-rc1
@@ -956,6 +977,11 @@ def ctx_dir_of(kdamond_idx, context_idx):
     return os.path.join(
             kdamond_dir_of(kdamond_idx), 'contexts', '%s' % context_idx)
 
+def target_dir_of(kdamond_idx, context_idx, target_idx):
+    return os.path.join(
+            ctx_dir_of(kdamond_idx, context_idx), 'targets',
+            '%s' % target_idx)
+
 def schemes_dir_of(kdamond_idx, context_idx):
     return os.path.join(ctx_dir_of(kdamond_idx, context_idx), 'schemes')
 
@@ -1000,7 +1026,9 @@ def infer_damon_version():
                                 )])])]
     err = stage_kdamonds(kdamonds)
     if err is None:
-        if os.path.isfile(os.path.join(kdamond_dir_of(0), 'refresh_ms')):
+        if os.path.isfile(os.path.join(ctx_dir_of(0, 0), 'addr_unit')):
+            version = '>v6.17'
+        elif os.path.isfile(os.path.join(kdamond_dir_of(0), 'refresh_ms')):
             version = '>v6.16'
         elif os.path.isdir(os.path.join(scheme_dir_of(0, 0, 0), 'dests')):
             version = '>v6.16'
@@ -1071,6 +1099,8 @@ def update_supported_features():
             _damon.Kdamond(
                 state=None, pid=None, contexts=[
                     _damon.DamonCtx(
+                        targets=[_damon.DamonTarget(
+                            pid=None, regions=[])],
                         schemes=[_damon.Damos()])])]
     err = stage_kdamonds(kdamonds_for_feature_check)
     if err is not None:
@@ -1121,12 +1151,16 @@ def update_supported_features():
                          'stats', 'sz_ops_filter_passed')):
         feature_supports['sz_ops_filter_passed'] = True
 
+    ops_filters_dir = os.path.join(scheme_dir_of(0, 0, 0), 'ops_filters')
+    if not os.path.isdir(ops_filters_dir):
+        ops_filters_dir = os.path.join(scheme_dir_of(0, 0, 0), 'filters')
+
     if os.path.isfile(
-            os.path.join(scheme_dir_of(0, 0, 0), 'filters', '0', 'allow')):
+            os.path.join(ops_filters_dir, '0', 'allow')):
         feature_supports['allow_filter'] = True
 
     if os.path.isfile(
-            os.path.join(scheme_dir_of(0, 0, 0), 'filters', '0', 'min')):
+            os.path.join(ops_filters_dir, '0', 'min')):
         feature_supports['schemes_filters_hugepage_size'] = True
 
     if os.path.isdir(
@@ -1148,6 +1182,8 @@ def update_supported_features():
                 _damon.Kdamond(
                     state=None, pid=None, contexts=[
                         _damon.DamonCtx(
+                            targets=[_damon.DamonTarget(
+                                pid=None, regions=[])],
                             schemes=[_damon.Damos(
                                 quotas=_damon.DamosQuotas(
                                     goals=[_damon.DamosQuotaGoal()])
@@ -1163,12 +1199,23 @@ def update_supported_features():
                              'nid')):
             feature_supports['schemes_quota_goal_node_mem_used_free'] = True
 
+        if os.path.isfile(
+                os.path.join(scheme_dir_of(0, 0, 0), 'quotas', 'goals', '0',
+                             'path')):
+            feature_supports['schemes_quota_goal_node_memcg_used_free'] = True
+
     if os.path.isdir(os.path.join(scheme_dir_of(0, 0, 0), 'dests')):
         feature_supports['schemes_dests'] = True
 
     if os.path.isfile(os.path.join(kdamond_dir_of(0), 'refresh_ms')):
         feature_supports['sysfs_refresh_ms'] = True
 
+    if os.path.isfile(os.path.join(ctx_dir_of(0, 0), 'addr_unit')):
+        feature_supports['addr_unit'] = True
+
+    if os.path.isfile(os.path.join(target_dir_of(0, 0, 0), 'obsolete_target')):
+        feature_supports['obsolete_target'] = True
+
     if os.path.isdir(os.path.join(ctx_dir_of(0, 0), 'operations_attrs')):
         feature_supports['ops_attrs'] = True
 
