@@ -54,10 +54,12 @@ class HeatMap:
     addr_unit = None
     addr_resol = None
 
+    heat_unit = None
+
     pixels = None # list of list of pixels
 
     def __init__(self, time_start, time_unit, time_resol, addr_start,
-                 addr_unit, addr_resol):
+                 addr_unit, addr_resol, heat_unit):
         self.time_start = time_start
         self.time_unit = time_unit
         self.time_resol = time_resol
@@ -66,6 +68,8 @@ class HeatMap:
         self.addr_unit = addr_unit
         self.addr_resol = addr_resol
 
+        self.heat_unit = heat_unit
+
         self.pixels = []
         for i in range(time_resol):
             self.pixels.append([])
@@ -81,12 +85,13 @@ class HeatMap:
     def pixel_idx_of_addr(self, addr):
         return int((addr - self.addr_start) / self.addr_unit)
 
-    def add_pixel_heat(self, pixels_idx, pixel_idx, region, snapshot, aggr_ns,
-                       df_passed):
+    def add_pixel_heat(self, pixels_idx, pixel_idx, region, snapshot,
+                       record_intervals, df_passed):
         pixel = self.pixels[pixels_idx][pixel_idx]
         observe_start_time = snapshot.start_time
-        if aggr_ns is not None:
-            observe_start_time -= region.age.aggr_intervals * aggr_ns
+        if record_intervals is not None:
+            observe_start_time -= region.age.aggr_intervals * \
+                    record_intervals.aggr * 1000
 
         account_time_start = max(observe_start_time, pixel.time)
         account_time_end = min(snapshot.end_time, pixel.time + self.time_unit)
@@ -96,11 +101,16 @@ class HeatMap:
         account_addr_end = min(region.end, pixel.addr + self.addr_unit)
         account_sz = account_addr_end - account_addr_start
 
-        nr_accesses = region.nr_accesses.samples
+        if record_intervals is not None:
+            aggr_us = damo_report_access.snapshot_monitoring_intervals(
+                    snapshot, record_intervals)[1]
+            heat_val = region.nr_accesses.in_hz(aggr_us)
+        else:
+            heat_val = region.nr_accesses.samples
         if df_passed:
-            nr_accesses = nr_accesses * region.sz_filter_passed / region.size()
+            heat_val = heat_val * region.sz_filter_passed / region.size()
 
-        heat = nr_accesses * account_time * account_sz
+        heat = heat_val * account_time * account_sz
 
         if pixel.heat is None:
             pixel.heat = 0
@@ -120,7 +130,11 @@ class HeatMap:
                 self.pixels_idx_of_time(start_time),
                 self.pixels_idx_of_time(snapshot.end_time) + 1)
 
-    def add_heat(self, snapshot, last_snapshot, aggr_ns, df_passed):
+    def add_heat(self, snapshot, last_snapshot, record_intervals, df_passed):
+        if record_intervals is not None:
+            aggr_ns = record_intervals.aggr * 1000
+        else:
+            aggr_ns = None
         heatmap_addr_end = self.addr_start + self.addr_unit * self.addr_resol
         for region in snapshot.regions:
             if region.end < self.addr_start or heatmap_addr_end < region.start:
@@ -136,8 +150,8 @@ class HeatMap:
                     if pixel_idx < 0 or self.addr_resol <= pixel_idx:
                         continue
                     self.add_pixel_heat(
-                            pixels_idx, pixel_idx, region, snapshot, aggr_ns,
-                            df_passed)
+                            pixels_idx, pixel_idx, region, snapshot,
+                            record_intervals, df_passed)
 
     def highest_lowest_heats(self):
         highest = None
@@ -193,9 +207,13 @@ class HeatMap:
 
     def fmt_ascii_lines_comments(self, colorset, print_colorset):
         lines = []
+
         if print_colorset:
-            lines.append('# access_frequency: %s' %
-                    _damo_ascii_color.color_samples(colorset))
+            highest_heat, lowest_heat = self.highest_lowest_heats()
+            range_str = '%.3f-%.3f %s' % (lowest_heat, highest_heat,
+                                          self.heat_unit)
+            lines.append('# access_frequency: %s (%s)' %
+                    (_damo_ascii_color.color_samples(colorset), range_str))
 
         addr_start = self.addr_start
         addr_len = self.addr_unit * self.addr_resol
@@ -233,15 +251,19 @@ def heatmap_from_records(
     time_unit = (time_end - time_start) // time_resol
     addr_unit = (addr_end - addr_start) // addr_resol
 
+    heat_unit = 'samples'
+    for record in records:
+        if record.intervals is not None:
+            heat_unit = 'hz'
+            break
+
     heatmap = HeatMap(time_start, time_unit, time_resol,
-                      addr_start, addr_unit, addr_resol)
+                      addr_start, addr_unit, addr_resol, heat_unit)
     last_snapshot = None
     for record in records:
-        aggr_ns = None
-        if record.intervals is not None:
-            aggr_ns = record.intervals.aggr * 1000
         for snapshot in record.snapshots:
-            heatmap.add_heat(snapshot, last_snapshot, aggr_ns, df_passed)
+            heatmap.add_heat(snapshot, last_snapshot, record.intervals,
+                             df_passed)
             last_snapshot = snapshot
     return heatmap
 
@@ -420,7 +442,7 @@ def plot_range(orig_range, use_absolute_val):
     return plot_range
 
 def plot_heatmap(data_file, output_file, args, address_range, range_idx,
-                 highest_heat, lowest_heat):
+                 highest_heat, lowest_heat, access_frequency_unit):
     terminal = output_file.split('.')[-1]
     if not terminal in ['pdf', 'jpeg', 'png', 'svg']:
         os.remove(data_file)
@@ -447,10 +469,13 @@ def plot_heatmap(data_file, output_file, args, address_range, range_idx,
     set cbrange [%f:%f];
     set xlabel 'Time (ns)';
     set ylabel '%s';
+    set cblabel 'Access frequency (%s)';
+    set lmargin at screen 0.1;
+    set rmargin at screen 0.7;
     plot '%s' using 1:2:3 with image;""" % (
             terminal, output_file, x_range[0], x_range[1],
             y_range[0], y_range[1], highest_heat, lowest_heat, ylabel,
-            data_file)
+            access_frequency_unit, data_file)
     try:
         subprocess.call(['gnuplot', '-e', gnuplot_cmd])
     except Exception as e:
@@ -458,6 +483,12 @@ def plot_heatmap(data_file, output_file, args, address_range, range_idx,
     os.remove(data_file)
 
 def mk_show_heatmap(records, args):
+    access_frequency_unit = 'samples'
+    for record in records:
+        if record.intervals is not None:
+            access_frequency_unit = 'hz'
+            break
+
     heats_list = []
     for idx in range(len(args.address_range)):
         heatmap = mk_heatmap(args, idx, records)
@@ -479,7 +510,7 @@ def mk_show_heatmap(records, args):
                 f.write(gnuplot_data_str)
             plot_heatmap(
                     tmp_path, args.output, args, args.address_range[idx], idx,
-                    highest_heat, lowest_heat)
+                    highest_heat, lowest_heat, access_frequency_unit)
 
 def scale_range(start_end, ratio):
     start, end = start_end
@@ -602,6 +633,15 @@ def main(args):
                 (args.input, err))
         exit(1)
 
+    # old DAMON records may have None intervals, while newer ones would have
+    # none-None intervals.  The intervals are used for showing access frequency
+    # in hz.  If intervals are None, nr_accesses (samples) is just used.
+    # Reject mixed versions of records to make the output consistent.
+    nr_having_intervals = len([r for r in records if r.intervals is not None])
+    if not nr_having_intervals in [0, len(records)]:
+        print('Some records have intervals while some not.  Cannot proceed.')
+        exit(1)
+
     # Use 80x40 or 500x500 resolution as default for stdout or image plots
     if args.resol is None:
         if args.output == 'stdout':
